set mprint=no.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

DEFINE !UNIVAR (nrep = !charend('/')/
               STAT = !charend('/')/
               dep = !charend('/')/
               grp = !default(NOGRP) !charend('/')/
               wgt = !charend('/')/
               rwgt = !charend('/')/
               cons = !charend('/')/
               PSU = !charend('/')/
               limit_criteria = !default(0) !charend('/')/
               infile = !charend('/')).
preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables TEXTS /destination viewer=no /Tag="NoViewer".

/* nGrp is the number of group variables */
!if (!upcase(!grp)="NOGRP") !then
!let !nGrp=!null.
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).
!if (!ngrp=0) !then 
!let !x=2.
!else.
/* x is nGrp + 1 */
!let !x=!length(!concat(!blanks(!nGrp)," ")).
!ifend.
title !concat('(number of group variables is ',!nGrp,')(nGrp + 1 = ',!x,')').

dataset close all.
GET FILE !infile 
   /KEEP !if(!nGrp>0) !then !grp !ifend !PSU !dep !wgt !do !i= 1 !to !nrep !concat(!rwgt,!i) !doend.
!if(!nGrp=0) !then compute !grp=1 !ifend.
alter type !grp (a10).
missing values !grp ().
fre !grp.
SORT CASES BY !grp.
compute MISS=0.
!do !d !in (!dep).
if (missing(!d)) MISS=1.
!doend.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\a.sav' /drop=MISS.


*** COMPUTE ESTIMATE ***.

oms /select tables /if subtypes='Report' /destination format=sav outfile='c:\temp\fullwgt.sav' 
   /COLUMNS DIMNAMES = ["Variables"] /tag='FullWeight'.
WEIGHT BY !wgt.
means !dep !if (!nGrp>0) !then !do !g !in (!grp) !concat("by ",!g) !doend !else by !grp !ifend /cells=!STAT.
omsend tag=['FullWeight'].


* REPLICATES.   

oms /select tables /if subtypes='Report' /destination format=sav numbered="RepWeight" outfile='c:\temp\repwgt.sav' 
   /COLUMNS DIMNAMES = ["Variables"] /tag='RepWeight'.
!do !i= 1 !to !nrep.
WEIGHT BY !concat(!rwgt,!i).
means !dep !if (!nGrp>0) !then !do !g !in (!grp) !concat("by ",!g) !doend !else by !grp !ifend /cells=!STAT.
!doend.
omsend tag=['RepWeight'].


*** COMBINE FILES ***.

!if (!ngrp=0) !then !let !ngrp=1 !ifend.

get file='c:\temp\repwgt.sav' .
sort cases by !do !n=1 !to !nGrp !concat("Var",!n) !doend !concat(" Var",!x).
rename vars (!dep=!do !d !in (!dep) !concat("R_",!d) !doend).
save outfile='c:\temp\repwgt2.sav'.

get file='c:\temp\fullwgt.sav' .
sort cases by !do !n=1 !to !nGrp !concat("Var",!n) !doend !concat(" Var",!x).

match files table=* /file='c:\temp\repwgt2.sav' /by !do !n=1 !to !nGrp !concat(" Var",!n) !doend !concat(" Var",!x).
exe.

rename vars (!concat("Var",!x)=statistic).
!do !s !in (!STAT)
!if (!upcase(!s)=MEAN) !then recode statistic ("Mean"="MEAN") !ifend.
!if (!upcase(!s)=STDDEV) !then recode statistic("Std. Deviation"="STDDEV") !ifend.
!if (!upcase(!s)=MEDIAN) !then recode statistic("Median"="MEDIAN") !ifend.
!if (!upcase(!s)=SKEW) !then recode statistic("Skewness"="SKEW") !ifend.
!if (!upcase(!s)=VARIANCE) !then recode statistic("Variance"="VARIANCE") !ifend.
!if (!upcase(!s)=KURT) !then recode statistic("Kurtosis"="KURT") !ifend.
!doend.

!if (!nGrp > 0) !Then
rename vars (!do !n=1 !to !nGrp !concat("Var",!n) !doend =!grp).
!Ifend.

* Delete totals (Note: This might not work in all languages).
!do !g !in (!grp).
select if (substr(!g,1,3)<>"Tot").
!doend.
alter type !grp (a10).


*** COMPUTE SAMPLING VARIANCE (U) ***.

!do !d !in(!dep) 
COMPUTE !concat('var_',!d) = !cons*((!concat('r_',!d) - !d)**2)*10000.   
!doend.

AGGREGATE OUTFILE =*
   /PRESORTED /BREAK=!if (!nGrp > 0) !then !grp !ifend statistic 
   /!do !d !in(!dep) !d !doend=MEAN(!do !d !in(!dep) !d !doend) 
   /!do !d !in(!dep) !concat('var_',!d) !doend=SUM(!do !d !in(!dep) !concat('var_',!d) !doend) .


*** COMPUTE SE ***.

!do !d !in(!dep) 
COMPUTE !concat('se_',!d) = SQRT(!concat('var_',!d)/10000. ).
FORMATS !d (F8.3) / !concat('se_',!d) (F10.6).
!doend .

save outfile='c:\temp\z.sav'.


*** COMPUTE N ***.

get file='c:\temp\a.sav'.
filter by VALIDN.
weight by !wgt.
aggregate outfile='c:\temp\y.sav' /break=!grp /NU_cases=NU /N_cases=N.
weight off.
aggregate outfile=* /break=!grp !PSU /temp=N.
aggregate outfile=* /break=!grp /NU_psu=N.
filter off.
match files /file='c:\temp\y.sav' /file=* /by !grp.
match files file='c:\temp\z.sav' /TABLE=* /by !grp.
omsend Tag="NoViewer".


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_cases=(NU_cases<!b).
formats FLAG_cases (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_psu=(NU_psu<!b).
formats FLAG_psu (f1.0).
!ifend.
!doend.
exe.
!ifend.


*** OUTPUT TABLES ***.

OMS /SELECT TABLES headings
   /IF COMMANDS = ["Summarize"] SUBTYPES = ["Case Processing Summary"]
   /DESTINATION   VIEWER = NO /TAG = "SumNoCaseProc".
!do !s !in (!STAT).
Temp.
Select if (statistic=!quote(!upcase(!s))).

SUMMARIZE
  /TABLES= !if (!upcase(!grp)="NOGRP") !then !else !grp !ifend statISTIC !do !d !in(!dep) !d !concat('SE_',!d) !doend 
  !if (!limit_criteria=0) !then N_cases NU_cases NU_psu !else FLAG_cases FLAG_psu  !ifend
  /FORMAT= !if (!nGrp > 0) !then LIST NOCASENUM TOTAL !else LIST NOCASENUM !ifend
  !if (!nGrp=0) !then /TITLE=!quote(!concat(!upcase(!s),' and SE of ',!dep))
  !else /TITLE=!quote(!concat(!upcase(!s),' and SE of ',!dep,' by ',!grp)) !ifend
  /CELLS=NONE.
!doend 

OMSEND TAG=["SumNoCaseProc"].

var lab  
   NU_cases "No of unweighted cases included in analysis" 
   N_cases "No of weighted cases included in analysis"
   NU_psu "No of unweighted primary sampling units in the analysis".

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !l !in (!limit_criteria)
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then
var lab FLAG_cases !quote(!concat("Flag if NU_cases < ",!l," ")).
!else
var lab FLAG_psu !quote(!concat("Flag if NU_psu < ",!l," ")).
!ifend !doend !ifend.

delete variables !if (!ngrp=0) !then !grp !ifend !do !d !in(!dep) !concat(" var_",!d) !doend.
set mprint=no.
restore.
!enddefine.

